;==============================================================================
    include \masm32\include\masm32rt.inc
;==============================================================================

;-------------------------------------------------------------------
; The DLGTEMPLATE structure is declared incorrectly in windows.inc,
; causing the structure size to be 20 bytes instead of the 18 bytes
; that the Windows API expects:
;
; DLGTEMPLATE STRUCT DWORD
;  style             DWORD     ?
;  dwExtendedStyle   DWORD     ?
;  cdit              WORD      ?
;  x                 WORD      ?
;  y                 WORD      ?
;  lx                WORD      ?
;  ly                WORD      ?
; DLGTEMPLATE ENDS
;
; The corrected declaration below has been renamed to avoid a
; conflict.
;-------------------------------------------------------------------

_DLGTEMPLATE STRUCT
  style             DWORD ?
  dwExtendedStyle   DWORD ?
  cdit              WORD  ?
  x                 WORD  ?
  y                 WORD  ?
  lx                WORD  ?
  ly                WORD  ?
_DLGTEMPLATE ENDS

;==============================================================================
    .data

      hInstance dd 0
      hDlg      dd  0
      msg       MSG <>

      ;----------------------------------------------------
      ; For a modeless dialog the WS_VISIBLE style muat be
      ; specified for the dialog to be visible.
      ;----------------------------------------------------

      STYLE = WS_OVERLAPPEDWINDOW or WS_VISIBLE or DS_CENTER

      ;------------------------------------------------------
      ; Note that for a dialog the coordinates and width and
      ; height are in Dialog Themplate Units, not pixels.
      ;------------------------------------------------------

      dlgTemplate _DLGTEMPLATE < STYLE,  \  ; style
                                 0,      \  ; extended style
                                 0,      \  ; control count
                                 0,      \  ; X coordinate
                                 0,      \  ; Y coordinate
                                 100,    \  ; width
                                 75      \  ; height
                                 >

      ;-----------------------------------------------------
      ; The first 3 variable-length arrays must be present.
      ;
      ; Specifying zero for the class will cause the dialog
      ; to use the system-defined class.
      ;-----------------------------------------------------

      arrayMenu   dw 0                      ; no menu
      arrayClass  dw 0                      ; no class
      arrayTitle  dw 'T','e','s','t',0      ; title as unicode

    .code
;==============================================================================

DlgProc proc hwndDlg:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

    .if uMsg == WM_INITDIALOG

        invoke MessageBoxW, hwndDlg,
                            addr arrayTitle,
                            addr arrayTitle,
                            0

    .elseif uMsg == WM_CLOSE

        ;--------------------------------------------------------------
        ; A modal dialog is destroyed with the EndDialog function, but
        ; a modeless dialog must be destroyed with the DestroyWindow
        ; function, the same as would be done with a normal application
        ; window.
        ;--------------------------------------------------------------

        invoke DestroyWindow, hwndDlg

        ;------------------------------------------------------------
        ; For a modeless dialog, as for a normal application window,
        ; the WM_DESTROY message must be handled for the spplication
        ; to terminate correctly. Posting a WM_QUIT message to the
        ; message queue will cause the message loop to exit and
        ; return control to the system.
        ;------------------------------------------------------------

    .elseif uMsg == WM_DESTROY

        invoke PostQuitMessage, NULL

    .endif

    ;---------------------------------------------------------------
    ; Returning FALSE here will work for most messages. For the few
    ; where it will not work, return TRUE directly from the message
    ; handler code.
    ;---------------------------------------------------------------

    xor eax, eax
    ret

DlgProc endp

;==============================================================================
start:
;==============================================================================

    invoke GetModuleHandle, NULL
    mov hInstance, eax

    invoke CreateDialogIndirectParam, hInstance,
                                      addr dlgTemplate,
                                      0,                ; no parent
                                      addr DlgProc,
                                      0
    mov hDlg, eax

    ;--------------------------------------------------
    ; A modeless dialog must implement a message loop.
    ;--------------------------------------------------

  msgLoop:

    invoke GetMessage, ADDR msg, 0, 0, 0
    .if eax != 0
        invoke IsDialogMessage, hDlg, ADDR msg
        jmp msgLoop
    .endif

    exit

;==============================================================================
end start